import { describe, expect, it, vi, beforeEach, afterEach } from 'vitest';
import { agentStates } from '../tools/interaction/agentStart.js';
import { processStates } from '../tools/system/shellStart.js';
import { BackgroundTools, BackgroundToolStatus } from './backgroundTools';
// Mock dependencies
vi.mock('../tools/browser/BrowserManager.js', () => {
    return {
        BrowserManager: class MockBrowserManager {
            closeSession = vi.fn().mockResolvedValue(undefined);
        },
    };
});
vi.mock('../tools/system/shellStart.js', () => {
    return {
        processStates: new Map(),
    };
});
vi.mock('../tools/interaction/agentStart.js', () => {
    return {
        agentStates: new Map(),
    };
});
describe('BackgroundTools cleanup', () => {
    let backgroundTools;
    // Setup mocks for globalThis and process states
    beforeEach(() => {
        backgroundTools = new BackgroundTools('test-agent');
        // Reset mocks
        vi.resetAllMocks();
        // Setup global browser manager
        globalThis.__BROWSER_MANAGER__ = {
            closeSession: vi.fn().mockResolvedValue(undefined),
        };
        // Setup mock process states
        const mockProcess = {
            kill: vi.fn(),
        };
        const mockProcessState = {
            process: mockProcess,
            state: { completed: false },
            command: 'test command',
            stdout: [],
            stderr: [],
            showStdIn: false,
            showStdout: false,
        };
        processStates.clear();
        processStates.set('shell-1', mockProcessState);
        // Setup mock agent states
        const mockAgentState = {
            aborted: false,
            completed: false,
            context: {
                backgroundTools: {
                    cleanup: vi.fn().mockResolvedValue(undefined),
                },
            },
            goal: 'test goal',
            prompt: 'test prompt',
            output: '',
            workingDirectory: '/test',
            tools: [],
        };
        agentStates.clear();
        agentStates.set('agent-1', mockAgentState);
    });
    afterEach(() => {
        vi.resetAllMocks();
        // Clear global browser manager
        globalThis.__BROWSER_MANAGER__ = undefined;
        // Clear mock states
        processStates.clear();
        agentStates.clear();
    });
    it('should clean up browser sessions', async () => {
        // Register a browser tool
        const browserId = backgroundTools.registerBrowser('https://example.com');
        // Run cleanup
        await backgroundTools.cleanup();
        // Check that closeSession was called
        expect(globalThis
            .__BROWSER_MANAGER__.closeSession).toHaveBeenCalledWith(browserId);
        // Check that tool status was updated
        const tool = backgroundTools.getToolById(browserId);
        expect(tool?.status).toBe(BackgroundToolStatus.COMPLETED);
    });
    it('should clean up shell processes', async () => {
        // Register a shell tool
        const shellId = backgroundTools.registerShell('echo "test"');
        // Get mock process state
        const mockProcessState = processStates.get('shell-1');
        // Set the shell ID to match
        processStates.set(shellId, processStates.get('shell-1'));
        // Run cleanup
        await backgroundTools.cleanup();
        // Check that kill was called
        expect(mockProcessState?.process.kill).toHaveBeenCalledWith('SIGTERM');
        // Check that tool status was updated
        const tool = backgroundTools.getToolById(shellId);
        expect(tool?.status).toBe(BackgroundToolStatus.COMPLETED);
    });
    it('should clean up sub-agents', async () => {
        // Register an agent tool
        const agentId = backgroundTools.registerAgent('Test goal');
        // Get mock agent state
        const mockAgentState = agentStates.get('agent-1');
        // Set the agent ID to match
        agentStates.set(agentId, agentStates.get('agent-1'));
        // Run cleanup
        await backgroundTools.cleanup();
        // Check that agent was marked as aborted
        expect(mockAgentState?.aborted).toBe(true);
        expect(mockAgentState?.completed).toBe(true);
        // Check that cleanup was called on the agent's background tools
        expect(mockAgentState?.context.backgroundTools.cleanup).toHaveBeenCalled();
        // Check that tool status was updated
        const tool = backgroundTools.getToolById(agentId);
        expect(tool?.status).toBe(BackgroundToolStatus.TERMINATED);
    });
    it('should handle errors during cleanup', async () => {
        // Register a browser tool
        const browserId = backgroundTools.registerBrowser('https://example.com');
        // Make closeSession throw an error
        globalThis
            .__BROWSER_MANAGER__.closeSession.mockRejectedValue(new Error('Test error'));
        // Run cleanup
        await backgroundTools.cleanup();
        // Check that tool status was updated to ERROR
        const tool = backgroundTools.getToolById(browserId);
        expect(tool?.status).toBe(BackgroundToolStatus.ERROR);
        expect(tool?.metadata.error).toBe('Test error');
    });
    it('should only clean up running tools', async () => {
        // Register a browser tool and mark it as completed
        const browserId = backgroundTools.registerBrowser('https://example.com');
        backgroundTools.updateToolStatus(browserId, BackgroundToolStatus.COMPLETED);
        // Run cleanup
        await backgroundTools.cleanup();
        // Check that closeSession was not called
        expect(globalThis
            .__BROWSER_MANAGER__.closeSession).not.toHaveBeenCalled();
    });
});
//# sourceMappingURL=backgroundTools.cleanup.test.js.map