export declare enum BackgroundToolType {
    SHELL = "shell",
    BROWSER = "browser",
    AGENT = "agent"
}
export declare enum BackgroundToolStatus {
    RUNNING = "running",
    COMPLETED = "completed",
    ERROR = "error",
    TERMINATED = "terminated"
}
export interface BackgroundTool {
    id: string;
    type: BackgroundToolType;
    status: BackgroundToolStatus;
    startTime: Date;
    endTime?: Date;
    metadata: Record<string, any>;
}
export interface ShellBackgroundTool extends BackgroundTool {
    type: BackgroundToolType.SHELL;
    metadata: {
        command: string;
        exitCode?: number | null;
        signaled?: boolean;
        error?: string;
    };
}
export interface BrowserBackgroundTool extends BackgroundTool {
    type: BackgroundToolType.BROWSER;
    metadata: {
        url?: string;
        error?: string;
    };
}
export interface AgentBackgroundTool extends BackgroundTool {
    type: BackgroundToolType.AGENT;
    metadata: {
        goal?: string;
        error?: string;
    };
}
export type AnyBackgroundTool = ShellBackgroundTool | BrowserBackgroundTool | AgentBackgroundTool;
/**
 * Registry to keep track of all background processes
 */
export declare class BackgroundTools {
    readonly ownerName: string;
    tools: Map<string, AnyBackgroundTool>;
    constructor(ownerName: string);
    registerShell(command: string): string;
    registerBrowser(url?: string): string;
    registerAgent(goal?: string): string;
    updateToolStatus(id: string, status: BackgroundToolStatus, metadata?: Record<string, any>): boolean;
    getTools(): AnyBackgroundTool[];
    getToolById(id: string): AnyBackgroundTool | undefined;
    /**
     * Cleans up all resources associated with this agent instance
     * @returns A promise that resolves when cleanup is complete
     */
    cleanup(): Promise<void>;
    /**
     * Cleans up a browser session
     * @param tool The browser tool to clean up
     */
    private cleanupBrowserSession;
    /**
     * Cleans up a shell process
     * @param tool The shell tool to clean up
     */
    private cleanupShellProcess;
    /**
     * Cleans up a sub-agent
     * @param tool The agent tool to clean up
     */
    private cleanupSubAgent;
}
//# sourceMappingURL=backgroundTools.d.ts.map