import { v4 as uuidv4 } from 'uuid';
import { agentStates } from '../tools/interaction/agentStart.js';
import { processStates } from '../tools/system/shellStart.js';
// Types of background processes we can track
export var BackgroundToolType;
(function (BackgroundToolType) {
    BackgroundToolType["SHELL"] = "shell";
    BackgroundToolType["BROWSER"] = "browser";
    BackgroundToolType["AGENT"] = "agent";
})(BackgroundToolType || (BackgroundToolType = {}));
// Status of a background process
export var BackgroundToolStatus;
(function (BackgroundToolStatus) {
    BackgroundToolStatus["RUNNING"] = "running";
    BackgroundToolStatus["COMPLETED"] = "completed";
    BackgroundToolStatus["ERROR"] = "error";
    BackgroundToolStatus["TERMINATED"] = "terminated";
})(BackgroundToolStatus || (BackgroundToolStatus = {}));
/**
 * Registry to keep track of all background processes
 */
export class BackgroundTools {
    ownerName;
    tools = new Map();
    // Private constructor for singleton pattern
    constructor(ownerName) {
        this.ownerName = ownerName;
    }
    // Register a new shell process
    registerShell(command) {
        const id = uuidv4();
        const tool = {
            id,
            type: BackgroundToolType.SHELL,
            status: BackgroundToolStatus.RUNNING,
            startTime: new Date(),
            metadata: {
                command,
            },
        };
        this.tools.set(id, tool);
        return id;
    }
    // Register a new browser process
    registerBrowser(url) {
        const id = uuidv4();
        const tool = {
            id,
            type: BackgroundToolType.BROWSER,
            status: BackgroundToolStatus.RUNNING,
            startTime: new Date(),
            metadata: {
                url,
            },
        };
        this.tools.set(id, tool);
        return id;
    }
    // Register a new agent process (for future use)
    registerAgent(goal) {
        const id = uuidv4();
        const tool = {
            id,
            type: BackgroundToolType.AGENT,
            status: BackgroundToolStatus.RUNNING,
            startTime: new Date(),
            metadata: {
                goal,
            },
        };
        this.tools.set(id, tool);
        return id;
    }
    // Update the status of a process
    updateToolStatus(id, status, metadata) {
        const tool = this.tools.get(id);
        if (!tool) {
            return false;
        }
        tool.status = status;
        if (status === BackgroundToolStatus.COMPLETED ||
            status === BackgroundToolStatus.ERROR ||
            status === BackgroundToolStatus.TERMINATED) {
            tool.endTime = new Date();
        }
        if (metadata) {
            tool.metadata = { ...tool.metadata, ...metadata };
        }
        return true;
    }
    getTools() {
        const result = [];
        for (const tool of this.tools.values()) {
            result.push(tool);
        }
        return result;
    }
    // Get a specific process by ID
    getToolById(id) {
        return this.tools.get(id);
    }
    /**
     * Cleans up all resources associated with this agent instance
     * @returns A promise that resolves when cleanup is complete
     */
    async cleanup() {
        const tools = this.getTools();
        // Group tools by type for better cleanup organization
        const browserTools = tools.filter((tool) => tool.type === BackgroundToolType.BROWSER &&
            tool.status === BackgroundToolStatus.RUNNING);
        const shellTools = tools.filter((tool) => tool.type === BackgroundToolType.SHELL &&
            tool.status === BackgroundToolStatus.RUNNING);
        const agentTools = tools.filter((tool) => tool.type === BackgroundToolType.AGENT &&
            tool.status === BackgroundToolStatus.RUNNING);
        // Create cleanup promises for each resource type
        const browserCleanupPromises = browserTools.map((tool) => this.cleanupBrowserSession(tool));
        const shellCleanupPromises = shellTools.map((tool) => this.cleanupShellProcess(tool));
        const agentCleanupPromises = agentTools.map((tool) => this.cleanupSubAgent(tool));
        // Wait for all cleanup operations to complete in parallel
        await Promise.all([
            ...browserCleanupPromises,
            ...shellCleanupPromises,
            ...agentCleanupPromises,
        ]);
    }
    /**
     * Cleans up a browser session
     * @param tool The browser tool to clean up
     */
    async cleanupBrowserSession(tool) {
        try {
            const browserManager = globalThis.__BROWSER_MANAGER__;
            if (browserManager) {
                await browserManager.closeSession(tool.id);
            }
            this.updateToolStatus(tool.id, BackgroundToolStatus.COMPLETED);
        }
        catch (error) {
            this.updateToolStatus(tool.id, BackgroundToolStatus.ERROR, {
                error: error instanceof Error ? error.message : String(error),
            });
        }
    }
    /**
     * Cleans up a shell process
     * @param tool The shell tool to clean up
     */
    async cleanupShellProcess(tool) {
        try {
            const processState = processStates.get(tool.id);
            if (processState && !processState.state.completed) {
                processState.process.kill('SIGTERM');
                // Force kill after a short timeout if still running
                await new Promise((resolve) => {
                    setTimeout(() => {
                        try {
                            if (!processState.state.completed) {
                                processState.process.kill('SIGKILL');
                            }
                        }
                        catch {
                            // Ignore errors on forced kill
                        }
                        resolve();
                    }, 500);
                });
            }
            this.updateToolStatus(tool.id, BackgroundToolStatus.COMPLETED);
        }
        catch (error) {
            this.updateToolStatus(tool.id, BackgroundToolStatus.ERROR, {
                error: error instanceof Error ? error.message : String(error),
            });
        }
    }
    /**
     * Cleans up a sub-agent
     * @param tool The agent tool to clean up
     */
    async cleanupSubAgent(tool) {
        try {
            const agentState = agentStates.get(tool.id);
            if (agentState && !agentState.aborted) {
                // Set the agent as aborted and completed
                agentState.aborted = true;
                agentState.completed = true;
                // Clean up resources owned by this sub-agent
                await agentState.context.backgroundTools.cleanup();
            }
            this.updateToolStatus(tool.id, BackgroundToolStatus.TERMINATED);
        }
        catch (error) {
            this.updateToolStatus(tool.id, BackgroundToolStatus.ERROR, {
                error: error instanceof Error ? error.message : String(error),
            });
        }
    }
}
//# sourceMappingURL=backgroundTools.js.map